@@ -1,10 +1,10 @@
 package Net::DNS::Header;
 
 #
-# $Id: Header.pm 1360 2015-06-15 09:58:53Z willem $
+# $Id: Header.pm 1381 2015-08-25 07:36:09Z willem $
 #
 use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1360 $)[1];
+$VERSION = (qw$LastChangedRevision: 1381 $)[1];
 
 
 =head1 NAME
@@ -70,7 +70,7 @@ sub string {
 	my $ar	   = $self->arcount;
 
 	my $opt = $$self->edns;
-	my $edns = $opt->defined ? $opt->string : '';
+	my $edns = $opt->_specified ? $opt->string : '';
 
 	return <<END . $edns if $opcode eq 'UPDATE';
 ;;	id = $id
@@ -97,7 +97,7 @@ END
 }
 
 
-=head2 string
+=head2 print
 
     $packet->header->print;
 
@@ -163,7 +163,7 @@ sub rcode {
 		my $opt = $$self->edns;
 		unless ( defined $arg ) {
 			my $rcode = $opt->rcode;
-			return rcodebyval( $_ & 0x0f ) unless $opt->defined;
+			return rcodebyval( $_ & 0x0f ) unless $opt->_specified;
 			$rcode = ( $rcode & 0xff0 ) | ( $_ & 0x00f );
 			$opt->rcode($rcode);			# write back full 12-bit rcode
 			return $rcode == 16 ? 'BADVERS' : rcodebyval($rcode);
