@@ -1,10 +1,10 @@
 package Net::DNS::RR::TSIG;
 
 #
-# $Id: TSIG.pm 1446 2016-01-26 10:21:56Z willem $
+# $Id: TSIG.pm 1472 2016-04-04 21:49:18Z willem $
 #
 use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1446 $)[1];
+$VERSION = (qw$LastChangedRevision: 1472 $)[1];
 
 
 use strict;
@@ -21,10 +21,10 @@ use integer;
 
 use Carp;
 
-eval { require Digest::HMAC };
-eval { require Digest::MD5 };
-eval { require Digest::SHA };
-eval { require MIME::Base64 };
+eval 'require Digest::HMAC';
+eval 'require Digest::MD5';
+eval 'require Digest::SHA';
+eval 'require MIME::Base64';
 
 use Net::DNS::DomainName;
 use Net::DNS::Parameters;
@@ -33,6 +33,7 @@ use constant ANY  => classbyname qw(ANY);
 use constant TSIG => typebyname qw(TSIG);
 
 {
+	# source: http://www.iana.org/assignments/tsig-algorithm-names
 	my @algbyname = (
 		'HMAC-MD5.SIG-ALG.REG.INT' => 157,
 		'HMAC-SHA1'		   => 161,
@@ -347,7 +348,7 @@ sub sig_data {
 sub create {
 	my $class = shift;
 	my $karg  = shift;
-	croak 'argument missing or undefined' unless defined $karg;
+	croak 'argument undefined' unless defined $karg;
 
 	if ( ref($karg) ) {
 		if ( $karg->isa('Net::DNS::Packet') ) {
@@ -516,7 +517,7 @@ sub vrfyerrstr {
 
 				my ( $hash, @param ) = @{$digest{$digtype}};
 				my ( undef, @block ) = @param;
-				my $digest = new $hash(@param);
+				my $digest   = new $hash(@param);
 				my $function = sub {
 					my $hmac = new Digest::HMAC( shift, $digest, @block );
 					$hmac->add(shift);
@@ -850,4 +851,6 @@ DEALINGS IN THE SOFTWARE.
 
 L<perl>, L<Net::DNS>, L<Net::DNS::RR>, RFC2845, RFC4635
 
+<TSIG Algorithm Names|http://www.iana.org/assignments/tsig-algorithm-names>
+
 =cut
